import click
from .. import __version__
from .init import init


@click.group()
@click.version_option(__version__)
def main():
    pass


main.command()(init)


@main.command()
def install():
    print("install command not yet implemented")


@main.command()
def test():
    print("test command not yet implemented")



from pathlib import Path
import enum
import shutil


class CopyPolicy(enum.Enum):
    OVERWRITE = enum.auto()
    IGNORE = enum.auto()
    APPEND = enum.auto()


def copytree(src, dst, *, policy=CopyPolicy.IGNORE, mapping=None):
    """A generally forgiving variant of shutil.copytree that allows
    format-string style substitution to file names and contents.
    """
    if mapping is None:
        mapping = {}
    src = Path(src)
    dst = Path(dst)
    if not src.exists():
        raise FileNotFoundError(src)
    dst.mkdir(parents=True, exist_ok=True)
    for sub_src in src.glob('*'):
        if sub_src.name == '__pycache__':
            continue
        sub_dst = dst / sub_src.name.format(**mapping)
        if sub_src.is_dir():
            copytree(sub_src, sub_dst, policy=policy, mapping=mapping)
        else:
            sd_exists = sub_dst.exists()
            if sd_exists:
                if policy == CopyPolicy.OVERWRITE:
                    sub_dst.unlink()
                    sd_exists = False
                elif policy == CopyPolicy.IGNORE:
                    continue

            with sub_dst.open('at') as fobj_w:
                with sub_src.open('rt') as fobj_r:
                    fobj_w.write(fobj_r.read().format(**mapping))



import subprocess as sp


def git_variables(path, *variables):
    cmd = ['git', '-C', str(path), 'config']
    if len(variables) == 1:
        cmd.extend(['--get', variables[0]])
    else:
        cmd.append('-l')

    gitconfig = sp.run(cmd, check=True, stdout=sp.PIPE)

    stdout = gitconfig.stdout.decode()

    if len(variables) == 1:
        return {variables[0]: stdout.strip()}

    all_vars = dict(line.strip().split('=', 1)
                    for line in stdout.splitlines())
    return {var: all_vars[var] for var in variables}



import os
from pathlib import Path
import pytest
from click.testing import CliRunner
from .. import main


@pytest.mark.parametrize(
    "directory, organization, workflow, override, target, abort",
    [(None, 'org', 'wf', None, '.', True),
     ('.', 'org', 'wf', None, '.', True),
     (None, 'org', 'wf', 'y', '.', False),
     ('.', 'org', 'wf', 'y', '.', False),
     ('niflow-org-wf', 'org', 'wf', None, 'niflow-org-wf', False),
     ('org-wf', 'org', 'wf', None, 'niflow-org-wf', False),
     ('ow', 'org', 'wf', '', 'niflow-org-wf', False),
     ('ow', 'org', 'wf', 'y', 'niflow-org-wf', False),
     ('ow', 'org', 'wf', 'n', 'niflow-ow', False)])
def test_init_path_selection(directory, organization, workflow, override, target, abort):
    runner = CliRunner()

    args = ['init']
    inputs = []

    orig_path = Path(directory or '.')
    targ_path = Path(target)

    if directory is not None:
        args.append(directory)

    # Non-standard paths prompt user input
    if orig_path.name not in (f'niflow-{organization}-{workflow}',
                              f'{organization}-{workflow}'):
        inputs.extend([organization, workflow])
        if override is not None:
            inputs.append(override)

    with runner.isolated_filesystem():
        result = runner.invoke(main, args, '\n'.join(inputs))

        if abort:
            assert result.exit_code != 0
            assert not (targ_path / '.git').exists()
        else:
            assert result.exit_code == 0

            assert targ_path.exists()
            assert (targ_path / '.git').exists()


@pytest.mark.parametrize(
    "author, email, gitconfig",
    [('Test Author', 'unreal3214@fake2182.tld', os.devnull),
     ('Test Author', 'unreal3214@fake2182.tld', 'tmp_gitconfig')])
def test_init_python(author, email, gitconfig):
    runner = CliRunner()

    args = ['init', 'niflow-org-wf', '--language', 'python']

    with runner.isolated_filesystem():
        inputs = None
        if gitconfig != os.devnull:
            config_path = Path(gitconfig)
            config_path.write_text(f'[user]\n\temail = {email}\n\tname = {author}')
            gitconfig = str(config_path.absolute())
        else:
            inputs = '\n'.join([author, email])

        result = runner.invoke(main, args, inputs, env={'GIT_CONFIG': gitconfig})

        assert result.exit_code == 0

    if gitconfig == os.devnull:
        assert 'Enter package author name: ' in result.stdout
        assert 'Enter package author email: ' in result.stdout
    else:
        assert 'Enter package author name: ' not in result.stdout
        assert 'Enter package author email: ' not in result.stdout



import pytest
from click.testing import CliRunner
from .. import main


@pytest.mark.parametrize("option", ["--help", "--version"])
def test_utility_options(option):
    runner = CliRunner()
    result = runner.invoke(main, [option])
    assert result.exit_code == 0


@pytest.mark.parametrize("command", ["test", "install"])
def test_unimplemented(command):
    runner = CliRunner()
    result = runner.invoke(main, [command])
    assert result.exit_code == 0
    assert "not yet implemented" in result.stdout



#!/usr/bin/env python3
from pkg_resources import resource_filename as pkgr_fn
from pathlib import Path
import subprocess as sp
import click

from ..util.git import git_variables
from ..util.fsutil import copytree


def normalize_path(name):
    path = Path(name).absolute()
    full_name = path.name

    # Accept organization-workflow format and normalize to niflow-organization-workflow
    if name != '.' and not full_name.startswith('niflow-'):
        full_name = 'niflow-' + full_name
        path = path.with_name(full_name)

    # Make guesses as to organization and workflow name
    if full_name.startswith('niflow-'):
        name_parts = full_name.split('-', 2)[1:]
    else:
        name_parts = full_name.split('-', 1)

    if len(name_parts) == 2:
        # Normal name, do not prompt
        organization, workflow = name_parts
    else:
        # Ambiguous, ask for input
        organization = click.prompt("Organization name")
        workflow = click.prompt("Workflow name", default=name_parts[0])
        full_name = '-'.join(['niflow', organization, workflow])
        if name != '.':
            if click.confirm(f'Update path to {full_name}?', default=True):
                path = path.with_name(full_name)
        else:
            click.confirm(f'Niflow name "{full_name}" does not match directory "{path.name}". '
                          'Proceed anyway?', abort=True)

    return path, full_name, organization, workflow


@click.argument('name', type=click.Path(), default='.')
@click.option('--language', help='Language for new niflow')
def init(name, language):
    path, full_name, organization, workflow = normalize_path(name)

    click.echo(f'Initializing workflow: {path.name} in {path.parent}')
    path.mkdir(parents=True, exist_ok=True)
    sp.run(['git', '-C', str(path), 'init'], check=True)

    try:
        git_vars = git_variables(path, 'user.name', 'user.email')
    except KeyError:
        username = click.prompt("Enter package author name")
        email = click.prompt("Enter package author email")
    else:
        username = git_vars['user.name']
        email = git_vars['user.email']

    mapping = {
        'USERNAME': username,
        'USEREMAIL': email,
        'ORGANIZATION': organization,
        'WORKFLOW': workflow,
        'FULLNAME': full_name,
        }

    copytree(pkgr_fn('niflow_manager', 'data/templates/base'), path, mapping=mapping)

    if language is not None:
        language_dir = Path(pkgr_fn('niflow_manager', f'data/templates/{language}'))
        try:
            copytree(language_dir, path, mapping=mapping)
        except FileNotFoundError:
            raise ValueError(f"Unknown language: {language}")
